@@ -1,7 +1,11 @@
 from typing import Optional
 from abc import ABC, abstractmethod
 import tkinter as tk
+from tkinter import ttk
 from .Sound import Sound
+import re
+import sounddevice as sd
+import numpy as np
 
 class MouseMotionListener(ABC):
     @abstractmethod
@@ -35,10 +39,10 @@ class LineListener(ABC):
     def update(self, event):
         pass
 
-class SamplingPanel(tk.Frame):
+class SamplingPanel(tk.Frame,):
     """Class to display the sound wave."""
     
-    def __init__(self, parent, sound_explorer):
+    def __init__(self, parent, sound_explorer: 'SoundExplorer'):
         super().__init__(parent)
         self.sound_explorer = sound_explorer
         self.points = []
@@ -63,7 +67,6 @@ class SamplingPanel(tk.Frame):
             highlightthickness=0
         )
         self.canvas.pack(fill=tk.BOTH, expand=True)
-        
         # Bind mouse events to canvas
         self.canvas.bind('<Motion>', sound_explorer.mouse_moved)
         self.canvas.bind('<Button-1>', sound_explorer.mouse_clicked)
@@ -75,7 +78,7 @@ class SamplingPanel(tk.Frame):
     
     def create_wave_form(self):
         """Create the sound wave visualization."""
-        sound = self.sound_explorer.sound
+        sound: Sound = self.sound_explorer.sound
         sample_width = self.sound_explorer.sample_width
         sample_height = self.sound_explorer.sample_height
         frames_per_pixel = self.sound_explorer.frames_per_pixel
@@ -89,10 +92,6 @@ class SamplingPanel(tk.Frame):
             if hasattr(sound, 'get_audio_file_format'):
                 format_info = sound.get_audio_file_format()
                 sample_size_bits = format_info.get_sample_size_in_bits()
-            elif hasattr(sound, 'getSampleSize'):
-                sample_size_bits = sound.getSampleSize()
-            elif hasattr(sound, 'getBitsPerSample'):
-                sample_size_bits = sound.getBitsPerSample()
             else:
                 # Default to 16-bit
                 sample_size_bits = 16
@@ -118,7 +117,7 @@ class SamplingPanel(tk.Frame):
             try:
                 sample_index = int(pixel * frames_per_pixel)
                 if sample_index < sound.getLengthInFrames():
-                    sample_value = sound.getSample(sample_index)
+                    sample_value = sound.getSampleValue(sample_index)
                     # Normalize the sample value to fit in the display
                     y = (sample_height // 2) - (sample_value * (sample_height // 2) / max_value)
                     # Clamp y to valid range
@@ -158,6 +157,14 @@ class SamplingPanel(tk.Frame):
                 fill=self.sound_explorer.SELECTION_COLOR,
                 outline=""
             )
+
+        if (self.sound_explorer.selection_start != -1 and self.sound_explorer.selection_stop == -1):
+            self.canvas.create_rectangle(
+                self.sound_explorer.selection_start, 0,
+                self.sound_explorer.selection_start+2, self.sound_explorer.sample_height,
+                fill=self.sound_explorer.BAR_COLOR,
+                outline=""
+            )
         
         # Draw center line first (baseline)
         center_y = self.sound_explorer.sample_height // 2
@@ -192,7 +199,19 @@ class SamplingPanel(tk.Frame):
                 fill="red", 
                 font=("Arial", 12)
             )
-            
+        # Draw the current index bar (on top of waveform)
+        try:
+            x = int(self.sound_explorer.current_pixel_position)
+            if 0 <= x < self.sound_explorer.sample_width:
+                self.canvas.create_line(
+                    x, 0, x, self.sound_explorer.sample_height,
+                    fill=self.sound_explorer.BAR_COLOR,
+                    width=1
+                )
+        except Exception:
+            # If anything goes wrong drawing the bar, ignore and continue
+            pass
+
         # Force canvas update
         self.canvas.update_idletasks()
     
@@ -201,21 +220,22 @@ class SamplingPanel(tk.Frame):
         self.repaint()
 
 class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineListener):
-    # Class constants (equivalent to static final)
-    ZOOM_IN_HINT = "Click to see all the samples (the number of samples between pixels is 1)"
-    CURRENT_INDEX_TEXT = "Current Index: "
-    START_INDEX_TEXT = "Start Index: "
-    STOP_INDEX_TEXT = "Stop Index: "
-    SAMPLE_TEXT = "Sample Value: "
     
-    # Color constants
-    SELECTION_COLOR = "gray"
-    BACKGROUND_COLOR = "black"
-    WAVE_COLOR = "white"
-    BAR_COLOR = "cyan"
     
     def __init__(self, sound:Sound):
         """Initialize the SoundExplorer with a sound."""
+        # Class constants (equivalent to static final)
+        self.ZOOM_IN_HINT = "Click to see all the samples (the number of samples between pixels is 1)"
+        self.CURRENT_INDEX_TEXT = "Current Index: "
+        self.START_INDEX_TEXT = "Start Index: "
+        self.STOP_INDEX_TEXT = "Stop Index: "
+        self.SAMPLE_TEXT = "Sample Value: "
+    
+    # Color constants
+        self.SELECTION_COLOR = "grey"
+        self.BACKGROUND_COLOR = "black"
+        self.WAVE_COLOR = "white"
+        self.BAR_COLOR = "cyan"
         
         # Store the primary parameters
         self.sound = sound
@@ -233,13 +253,14 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         self.zoom_in_width: int = sound.getLengthInFrames()
         self.sample_width: int = self.zoom_out_width
         self.frames_per_pixel: float = sound.getLengthInFrames() / self.sample_width
+        self.MAX_FRAMES_PER_PIXEL = self.frames_per_pixel
         self.sample_height: int = 201
         
         # Current pixel position
         self.current_pixel_position: int = 0
         
         # Debug flag
-        self.debug: bool = True  # Enable debug for testing
+        self.debug: bool = False  # Enable debug for testing
         
         # Main parts of the GUI
         self.sound_frame: Optional[tk.Tk] = None
@@ -262,6 +283,7 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         # Parts of the sound panel
         self.sound_wrapper: Optional[tk.Frame] = None
         self.sample_panel = None  # SamplingPanel equivalent
+        self.scrollbar_h: Optional[ttk.Scrollbar] = None
         
         # Parts of the information panel
         self.info_panel: Optional[tk.Frame] = None
@@ -299,7 +321,17 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
     # Interface method implementations
     def mouse_clicked(self, event):
         """Handle mouse click event."""
-        self.current_pixel_position = event.x
+        # Translate to canvas coordinates in case canvas is scrolled
+        try:
+            canvas_x = int(self.sample_panel.canvas.canvasx(event.x))
+        except Exception:
+            canvas_x = event.x
+        self.current_pixel_position = canvas_x
+        # Clear any existing selection when the user single-clicks to place the bar
+        self.selection_start = -1
+        self.selection_stop = -1
+        self.start_index_label.config(text=self.START_INDEX_TEXT + "N/A")
+        self.stop_index_label.config(text=self.STOP_INDEX_TEXT + "N/A")
            
         if self.current_pixel_position == 0:
             self.play_before_button.config(state=tk.DISABLED)
@@ -316,44 +348,91 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
             
         self.update_index_values()
         self.sample_panel.update()
+        # Update play before/after based on new current position
+        cur_frame = int(self.current_pixel_position * self.frames_per_pixel) + self.base
+        if cur_frame <= 0:
+            self.play_before_button.config(state=tk.DISABLED)
+        else:
+            self.play_before_button.config(state=tk.NORMAL)
+
+        if cur_frame >= self.sound.getLengthInFrames() - 1:
+            self.play_after_button.config(state=tk.DISABLED)
+        else:
+            self.play_after_button.config(state=tk.NORMAL)
     
     def mouse_pressed(self, event):
         """Handle mouse press event."""
-        self.mouse_pressed_x = event.x
+        try:
+            self.mouse_pressed_x = int(self.sample_panel.canvas.canvasx(event.x))
+        except Exception:
+            self.mouse_pressed_x = event.x
         
     def mouse_released(self, event):
         """Handle mouse release event."""
-        self.mouse_released_x = event.x
+        try:
+            self.mouse_released_x = int(self.sample_panel.canvas.canvasx(event.x))
+        except Exception:
+            self.mouse_released_x = event.x
         
         if self.mouse_dragged_flag:
-            self.mouse_pressed_pos = self.mouse_pressed_x
-            self.mouse_released_pos = self.mouse_released_x
-                
-            if self.mouse_pressed_pos > self.mouse_released_pos:  # Selected right to left
-                self.mouse_pressed_pos, self.mouse_released_pos = self.mouse_released_pos, self.mouse_pressed_pos
-                
-            self.start_frame = int(self.mouse_pressed_pos * self.frames_per_pixel)
-            self.stop_frame = int(self.mouse_released_pos * self.frames_per_pixel)
-                
-            # Handle dragging outside the window
-            if self.stop_frame >= self.sound.getLengthInFrames():
-                self.stop_frame = self.sound.getLengthInFrames()
-                
-            if self.start_frame < 0:
-                self.start_frame = 0
-                
-            # Update labels
-            self.start_index_label.config(text=self.START_INDEX_TEXT + str(self.start_frame))
-            self.stop_index_label.config(text=self.STOP_INDEX_TEXT + str(self.stop_frame))
+            self.makeSelection()
+        else:
+            self.makeBar()
+
+    def makeSelection(self):
+        self.mouse_pressed_pos = self.mouse_pressed_x
+        self.mouse_released_pos = self.mouse_released_x
             
-            # For highlighting the selection
-            self.selection_start = self.mouse_pressed_pos
-            self.selection_stop = self.mouse_released_pos
+        if self.mouse_pressed_pos > self.mouse_released_pos:  # Selected right to left
+            self.mouse_pressed_pos, self.mouse_released_pos = self.mouse_released_pos, self.mouse_pressed_pos
+        
+        self.start_frame = int(self.mouse_pressed_pos * self.frames_per_pixel)
+        self.stop_frame = int(self.mouse_released_pos * self.frames_per_pixel)
                 
-            self.sample_panel.update()
-            self.play_selection_button.config(state=tk.NORMAL)
-            self.clear_selection_button.config(state=tk.NORMAL)
-            self.mouse_dragged_flag = False
+        # Handle dragging outside the window
+        if self.stop_frame >= self.sound.getLengthInFrames():
+            self.stop_frame = self.sound.getLengthInFrames()
+                
+        if self.start_frame < 0:
+            self.start_frame = 0
+                
+        # Update labels
+        self.start_index_label.config(text=self.START_INDEX_TEXT + str(self.start_frame))
+        self.stop_index_label.config(text=self.STOP_INDEX_TEXT + str(self.stop_frame))
+            
+        # For highlighting the selection
+        #self.selection_start = self.mouse_pressed_pos
+        #self.selection_stop = self.mouse_released_pos
+            
+        # Update current index to start frame (like JES)
+        self.current_pixel_position = self.mouse_pressed_pos
+                
+        #self.sample_panel.update()
+        self.play_selection_button.config(state=tk.NORMAL)
+        self.clear_selection_button.config(state=tk.NORMAL)
+        self.play_before_button.config(state=tk.NORMAL)
+        self.play_after_button.config(state=tk.NORMAL)
+        #self.mouse_dragged_flag = False
+            
+        # Update the index values to show the start frame
+        self.sample_panel.update()
+        self.update_index_values()
+        self.mouse_dragged_flag = False
+
+    def makeBar(self):
+        self.mouse_pressed_pos = self.mouse_pressed_x
+
+        self.start_frame = int(self.mouse_pressed_pos * self.frames_per_pixel)
+        self.start_index_label.config(text=self.START_INDEX_TEXT + str(self.start_frame))
+
+        self.selection_start = self.mouse_pressed_pos
+        self.current_pixel_position = self.mouse_pressed_pos
+        self.sample_panel.update()
+        self.play_before_button.config(state=tk.NORMAL)
+        self.play_after_button.config(state=tk.NORMAL)
+            
+        # Update the index values to show the start frame
+        self.update_index_values()
     
     def mouse_entered(self, event):
         """Handle mouse entered event."""
@@ -362,24 +441,42 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
     def mouse_exited(self, event):
         """Handle mouse exited event."""
         pass
+
+    def action_performed(self, event):
+        return super().action_performed(event)
+    
+    def mouse_moved(self, event):
+        return super().mouse_moved(event)
+    
+    def update(self, event):
+        return super().update(event)
         
     def mouse_dragged(self, event):
         """Handle mouse dragged event."""
         self.mouse_dragged_flag = True
         # Highlight the selection as we drag by simulating mouse release
-        self.mouse_released(event)
+        try:
+            self.mouse_released_x = int(self.sample_panel.canvas.canvasx(event.x))
+        #self.mouse_released(event)
+        except Exception:
+            self.mouse_released_x = event.x
+        self.update_selection_visual()
+
+    def update_selection_visual(self):
+        """Update just the visual selection highlight during drag."""
+        mouse_pressed_pos = self.mouse_pressed_x
+        mouse_released_pos = self.mouse_released_x
         
-    def mouse_moved(self, event):
-        """Handle mouse move event."""
-        pass
+        # Handle right-to-left selection for visual purposes
+        if mouse_pressed_pos > mouse_released_pos:
+            mouse_pressed_pos, mouse_released_pos = mouse_released_pos, mouse_pressed_pos
         
-    def update(self, event):
-        """Handle line event update."""
-        pass
-    
-    def action_performed(self, event):
-        """Handle action events (button clicks, etc.)."""
-        pass  # This is handled by action_performed_handler
+        # Update the selection coordinates for drawing
+        self.selection_start = mouse_pressed_pos
+        self.selection_stop = mouse_released_pos
+        
+        # Trigger visual update
+        self.sample_panel.update()
         
     def action_performed_handler(self, command: str):
         """Handle action events from buttons."""
@@ -387,47 +484,36 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
             if command == "Play Entire Sound":
                 self.sound.play()
             elif command == "Play Selection":
-                # Try different method names for playing selection
-                if hasattr(self.sound, 'playRange'):
-                    self.sound.playRange(1, self.start_frame, self.stop_frame)
-                elif hasattr(self.sound, 'play'):
-                    # If no range method, just play the whole sound
-                    print(f"Playing selection from {self.start_frame} to {self.stop_frame}")
-                    self.sound.play()
-            elif command == "Stop":
-                # Try different methods to stop playback
-                if hasattr(self.sound, 'stop'):
-                    self.sound.stop()
-                elif hasattr(self.sound, 'stopPlaying'):
-                    self.sound.stopPlaying()
+                if self.start_frame != self.stop_frame:
+                    self.sound.playRange(self.start_frame, self.stop_frame)
                 else:
-                    print("Stop not implemented")
+                    # No selection, play from current position
+                    current_frame = int(self.current_pixel_position * self.frames_per_pixel) + self.base
+                    self.sound.playRange(current_frame, self.sound.getLengthInFrames() - 1)
+            elif command == "Stop":
+                self.sound.stopPlaying()
             elif command == "Zoom In":
-                self.handle_zoom_in(True)
+                self.handle_zoom_in()
             elif command == "Zoom Out":
                 self.handle_zoom_out()
             elif command == "Play Before":
-                # Try different method names
-                if hasattr(self.sound, 'playRange'):
-                    self.sound.playRange(1, 0, int(self.current_pixel_position * self.frames_per_pixel))
-                elif hasattr(self.sound, 'PlayRange'):
-                    self.sound.PlayRange(1, 0, int(self.current_pixel_position * self.frames_per_pixel))
+                if self.selection_start != -1:
+                    self.sound.playRange(0, self.start_frame)
                 else:
-                    print(f"Playing before position {self.current_pixel_position}")
-                    self.sound.play()
+                    # Play before current position
+                    current_frame = int(self.current_pixel_position * self.frames_per_pixel) + self.base
+                    self.sound.playRange(0, current_frame)
             elif command == "Play After":
-                # Try different method names
-                if hasattr(self.sound, 'playRange'):
-                    self.sound.playRange(1, int(self.current_pixel_position * self.frames_per_pixel), self.sound.getLengthInFrames() - 1)
-                elif hasattr(self.sound, 'PlayRange'):
-                    self.sound.PlayRange(1, int(self.current_pixel_position * self.frames_per_pixel), self.sound.getLengthInFrames() - 1)
+                if self.selection_stop != -1:
+                    self.sound.playRange(self.stop_frame, self.sound.getLengthInFrames() - 1)
                 else:
-                    print(f"Playing after position {self.current_pixel_position}")
-                    self.sound.play()
+                    # Play after current position
+                    current_frame = int(self.current_pixel_position * self.frames_per_pixel) + self.base
+                    self.sound.playRange(current_frame, self.sound.getLengthInFrames() - 1)
         except Exception as ex:
             self.catch_exception(ex)
     
-    def handle_zoom_in(self, from_button: bool):
+    def handle_zoom_in(self):
         """Handle zoom in functionality."""
         if self.frames_per_pixel > 1:
             self.frames_per_pixel = max(1, self.frames_per_pixel // 2)
@@ -438,59 +524,86 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
             self.sample_panel.create_wave_form()
             self.update_index_values()
             self.sample_panel.update()
+            self.check_scroll()
+            self.update_zoom_buttons()
             
-            if self.frames_per_pixel == 1:
-                self.zoom_button.config(text="Zoom Out")
     
     def handle_zoom_out(self):
         """Handle zoom out - simplified."""
-        self.zoom_button.config(text="Zoom In")
         
-        self.sample_width = self.zoom_out_width
-        self.frames_per_pixel = self.sound.getLengthInFrames() / self.sample_width
+        # Increase frames_per_pixel (zooming out)
+        self.frames_per_pixel = min(self.MAX_FRAMES_PER_PIXEL, self.frames_per_pixel * 2)
+        self.sample_width = int(self.sound.getLengthInFrames() / self.frames_per_pixel)
         
-        if self.debug:
-            print(f"Zoom Out: currentPixelPosition = {self.current_pixel_position}")
-        
-        # Update panel sizes and recreate waveform
+        # Update display
         self._update_panel_sizes()
         self.sample_panel.create_wave_form()
-        
         self.update_index_values()
         self.sample_panel.update()
+        self.check_scroll()
+        self.update_zoom_buttons()
+
+    def update_zoom_buttons(self):
+        """Enable or disable zoom buttons based on current zoom level."""
+        can_zoom_in = self.frames_per_pixel > 1
+        can_zoom_out = self.frames_per_pixel < self.MAX_FRAMES_PER_PIXEL
+
+        if can_zoom_in:
+            self.zoom_in_button["state"] = tk.NORMAL
+        else:
+            self.zoom_in_button["state"] = tk.DISABLED
+        if can_zoom_out:
+            self.zoom_out_button["state"] = tk.NORMAL
+        else:
+            self.zoom_out_button["state"] = tk.DISABLED
+
     
     def check_scroll(self):
         """Check that the current position is in the viewing area and scroll if needed."""
-        # Only do this if we are not zoomed out
-        if self.sample_width != self.zoom_out_width:
-            # This would implement scrolling logic for tkinter
-            # More complex implementation needed for proper scrolling
-            pass
+        needs_scrollbar = self.sample_width > self.zoom_out_width
+    
+        if needs_scrollbar:
+            # Need scrollbar - create it if it doesn't exist
+            if self.scrollbar_h is None:
+                self.scrollbar_h = ttk.Scrollbar(self.sound_wrapper, orient=tk.HORIZONTAL)
+            self.scrollbar_h.pack(side=tk.BOTTOM, fill=tk.X)
+                
+            # Connect canvas to scrollbar
+            self.sample_panel.canvas.config(scrollregion=(0, 0, self.sample_width, self.sample_height))
+            self.sample_panel.canvas.config(xscrollcommand=self.scrollbar_h.set)
+            self.scrollbar_h.config(command=self.sample_panel.canvas.xview)
+            
+        else:
+            # Don't need scrollbar - just hide it, don't destroy it
+            if self.scrollbar_h is not None:
+                self.scrollbar_h.pack_forget()  # Hide instead of destroy
+                self.sample_panel.canvas.config(xscrollcommand=None)
+                self.sample_panel.canvas.config(scrollregion="")
     
     def handle_zoom_in_index(self, index: int):
         """Handle zoom in to view all sample values at specific index."""
-        if index % self.frames_per_pixel != 0:
-            # Do normal zoom in on current position
-            self.handle_zoom_in(False)
+        if self.frames_per_pixel > 1 and index % self.frames_per_pixel != 0:
+            self.handle_zoom_in()
             
-        # Change current position to the passed index
-        self.current_pixel_position = int(index / self.frames_per_pixel) - self.base
+        self.current_pixel_position = max(0, int((index - self.base) / self.frames_per_pixel))
             
-        # Check scroll and repaint
         self.check_scroll()
         self.sample_panel.update()
     
     def create_window(self):
         """Create and display the main window and all GUI components."""
         # Get filename for window title
-        file_name = self.sound.getFileName()
-        if file_name is None:
+        pattern = re.compile(r'([^\\/]+)[\\/][^\\/]+$')
+        match = pattern.search(self.sound.getFileName())
+        if match:
+            file_name = match.group(0)
+        else:
             file_name = "no file name"
         
         # Create main window
         self.sound_frame = tk.Tk()
         self.sound_frame.title(file_name)
-        
+        self.sound_frame.resizable(False, False)
         # Set up window properties
         self.sound_frame.protocol("WM_DELETE_WINDOW", self._on_window_close)
         
@@ -510,7 +623,7 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         self.sound_frame.update_idletasks()  # Calculate required size
         
         # Set a reasonable window size
-        window_width = max(self.sample_width + 20, 800)
+        window_width = self.sample_width + 20
         window_height = self.sample_height + 200  # Extra space for controls
         self.sound_frame.geometry(f"{window_width}x{window_height}")
         
@@ -526,7 +639,6 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
     def show(self):
         """Show the window and start the event loop. Call this after creating the SoundExplorer."""
         if self.sound_frame:
-            print("Starting tkinter mainloop...")
             try:
                 self.sound_frame.mainloop()
             except KeyboardInterrupt:
@@ -540,7 +652,6 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
                         self.sound_frame.destroy()
                 except:
                     pass
-                print("Window cleanup completed")
     
     def run(self):
         """Alias for show() method."""
@@ -548,7 +659,6 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
     
     def _on_window_close(self):
         """Handle window close event."""
-        print("Window closing...")
         # Try to stop any playing sounds
         try:
             if hasattr(self.sound, 'stop'):
@@ -597,12 +707,12 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         self.play_entire_button = self.make_button("Play Entire Sound", True, self.button_panel)
         self.selection_prev_state = False
         self.play_before_button = self.make_button("Play Before", False, self.button_panel)
-        self.play_after_button = self.make_button("Play After", True, self.button_panel)
+        self.play_after_button = self.make_button("Play After", False, self.button_panel)
         self.stop_button = self.make_button("Stop", True, self.button_panel)
         
         # Pack panels
-        self.button_panel.pack(side=tk.TOP, fill=tk.X)
-        selection_panel.pack(side=tk.BOTTOM, fill=tk.X)
+        self.button_panel.pack(side=tk.TOP, expand=True, anchor="center")
+        selection_panel.pack(side=tk.BOTTOM, expand=True, anchor="center")
     
     def create_sound_panel(self):
         """Create the panel that displays the sound waveform."""
@@ -626,10 +736,12 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         
         # Create zoom panel with zoom button
         self.zoom_button_panel = tk.Frame(self.info_panel)
-        self.zoom_button = self.make_button("Zoom In", True, self.zoom_button_panel)
+        
+
         
         index_panel.pack(side=tk.TOP, fill=tk.X)
         self.zoom_button_panel.pack(side=tk.BOTTOM, fill=tk.X)
+        self.zoom_button_panel.pack_configure(anchor="center")
     
     def setup_index_panel(self, index_panel: tk.Frame):
         """Set up the index panel with navigation buttons and value displays."""
@@ -646,7 +758,9 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         self.index_value.insert(0, str(self.base))
         self.index_value.bind('<Return>', self._index_value_changed)
         
-        self.sample_value = tk.Entry(top_panel, width=10, state='readonly')
+        self.sample_value = tk.Entry(top_panel, width=10)
+        self.sample_value.insert(0, self.sound.getSample(0).getValue() if self.sound.getLengthInFrames() > 0 else "N/A")
+        self.sample_value.config(state='readonly')
         
         # Create labels - simplified
         self.index_label = tk.Label(top_panel, text=self.CURRENT_INDEX_TEXT)
@@ -664,18 +778,32 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         self.next_button.pack(side=tk.LEFT, padx=2)
         self.last_button.pack(side=tk.LEFT, padx=2)
         
+        
         # Bottom panel for frames per pixel
-        bottom_panel = tk.Frame(index_panel)
-        frames_label = tk.Label(bottom_panel, text="Samples between pixels: ")
-        self.num_samples_per_pixel_field = tk.Entry(bottom_panel, width=10)
+        middle_panel = tk.Frame(index_panel)
+        frames_label = tk.Label(middle_panel, text="Samples between pixels: ")
+        self.num_samples_per_pixel_field = tk.Entry(middle_panel, width=10)
         self.num_samples_per_pixel_field.insert(0, str(int(self.frames_per_pixel)))
         self.num_samples_per_pixel_field.bind('<Return>', self._frames_per_pixel_changed)
-        
+        sample_rate_label = tk.Label(middle_panel, text=f"Sample Rate: {self.sound.sampleRate} Hz")
+        sample_rate_label.pack(side=tk.RIGHT, padx=5)
+
         frames_label.pack(side=tk.LEFT)
         self.num_samples_per_pixel_field.pack(side=tk.LEFT)
         
+        bottom_panel = tk.Frame(index_panel)
+
+        # Create zoom buttons
+        self.zoom_in_button = self.make_button("Zoom In", True, bottom_panel)
+        self.zoom_out_button = self.make_button("Zoom Out", False, bottom_panel)
+
+        self.zoom_in_button.pack(side=tk.LEFT, padx=10)
+        self.zoom_out_button.pack(side=tk.LEFT, padx=10)
+        
+
         # Pack panels
         top_panel.pack(side=tk.TOP, pady=2)
+        middle_panel.pack(side=tk.TOP, pady=5)
         bottom_panel.pack(side=tk.BOTTOM, pady=2)
         index_panel.pack(fill=tk.X)
     
@@ -690,19 +818,36 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         """Method to clear the selection information."""
         self.selection_start = -1
         self.selection_stop = -1
+        self.start_frame = 0
+        self.stop_frame = 0
         self.start_index_label.config(text=self.START_INDEX_TEXT + "N/A")
         self.stop_index_label.config(text=self.STOP_INDEX_TEXT + "N/A")
         self.sample_panel.update()
         self.play_selection_button.config(state=tk.DISABLED)
         self.clear_selection_button.config(state=tk.DISABLED)
+        self.play_before_button.config(state=tk.DISABLED)
+        self.play_after_button.config(state=tk.DISABLED)
     
     def update_index_values(self):
         """Method to update the index values to the current index position."""
         cur_frame = int(self.current_pixel_position * self.frames_per_pixel) + self.base
         
         # Update the display of the current sample (frame) index
-        self.index_value.delete(0, tk.END)
-        self.index_value.insert(0, str(cur_frame))
+        # Keep the index_value editable so users can type in a number.
+        # We update its contents but leave it in normal state.
+        try:
+            sel = self.index_value.selection_get()
+        except Exception:
+            sel = None
+        self.index_value.config(state='normal')
+        # self.index_value.delete(0, tk.END)
+        # self.index_value.insert(0, str(cur_frame))
+        # Restore selection if user had selected text
+        try:
+            if sel is not None:
+                self.index_value.selection_range(0, tk.END)
+        except Exception:
+            pass
         
         # Update the number of samples per (between) pixels field
         if self.num_samples_per_pixel_field is not None:
@@ -711,10 +856,10 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         
         # Try to update the value at the current sample index
         try:
-            sample = self.sound.getSample(cur_frame - self.base)
+            sample = self.sound.getSample(cur_frame - self.base).getValue()
             self.sample_value.config(state='normal')
             self.sample_value.delete(0, tk.END)
-            self.sample_value.insert(0, f"{sample:.4f}")
+            self.sample_value.insert(0, f"{sample:.0f}")
             self.sample_value.config(state='readonly')
         except Exception as ex:
             self.catch_exception(ex)
@@ -829,8 +974,42 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
         """Handle index value text field change."""
         try:
             index = int(self.index_value.get())
-            self.handle_zoom_in_index(index)
+            # Calculate the pixel position for this index
+            pixel_pos = int((index - self.base) / self.frames_per_pixel)
+            self.current_pixel_position = max(0, min(pixel_pos, self.sample_width - 1))
+            # Clear any selection when the index is set manually
+            self.selection_start = -1
+            self.selection_stop = -1
+            self.start_index_label.config(text=self.START_INDEX_TEXT + "N/A")
+            self.stop_index_label.config(text=self.STOP_INDEX_TEXT + "N/A")
+
+            # Do not auto-zoom when the user types an index. Just move the bar.
+            # If you want an explicit zoom-to-index action, call handle_zoom_in_index
+            # from a dedicated UI control.
             self.update_index_values()
+            self.check_scroll()
+            self.sample_panel.update()
+            canvas_width = self.sample_panel.canvas.winfo_width()
+            bar_x = self.current_pixel_position
+
+            scroll_region = self.sample_panel.canvas.bbox("all")
+            if scroll_region :
+                total_width = scroll_region[2] - scroll_region[0]
+                target_fraction = max(0, min((bar_x - canvas_width // 2) / total_width, 1))
+                self.sample_panel.canvas.xview_moveto(target_fraction)
+                self.sample_panel.update()
+
+            # Update play before/after based on new index
+            cur_frame = index
+            if cur_frame <= 0:
+                self.play_before_button.config(state=tk.DISABLED)
+            else:
+                self.play_before_button.config(state=tk.NORMAL)
+
+            if cur_frame >= self.sound.getLengthInFrames() - 1:
+                self.play_after_button.config(state=tk.DISABLED)
+            else:
+                self.play_after_button.config(state=tk.NORMAL)
         except ValueError:
             pass  # Invalid input, ignore
     
@@ -840,6 +1019,5 @@ class SoundExplorer(MouseMotionListener, ActionListener, MouseListener, LineList
             frames = int(self.num_samples_per_pixel_field.get())
             if frames > 0:
                 self.handle_frames_per_pixel(frames)
-                self.update_index_values()
         except ValueError:
             pass  # Invalid input, ignore
\ No newline at end of file
